@@ -627,11 +627,11 @@ translate_axes (ClutterInputDevice *device,
       switch (axis)
         {
         case CLUTTER_INPUT_AXIS_X:
-          retval[i] = x;
+          retval[i] = x / stage_x11->scale_factor;
           break;
 
         case CLUTTER_INPUT_AXIS_Y:
-          retval[i] = y;
+          retval[i] = y / stage_x11->scale_factor;
           break;
 
         default:
@@ -745,6 +745,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
   XGenericEventCookie *cookie;
   XIEvent *xi_event;
   XEvent *xevent;
+  int window_scale;
 
   backend_x11 = CLUTTER_BACKEND_X11 (clutter_get_default_backend ());
 
@@ -773,6 +774,8 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
   event->any.stage = stage;
 
+  window_scale = stage_x11->scale_factor;
+
   switch (xi_event->evtype)
     {
     case XI_HierarchyChanged:
@@ -821,8 +824,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         event->key.time = xev->time;
         event->key.stage = stage;
-        event->key.modifier_state =
-          _clutter_input_device_xi2_translate_state (&xev->mods, &xev->buttons, &xev->group);
+	_clutter_input_device_xi2_translate_state (event, &xev->mods, &xev->buttons, &xev->group);
         event->key.hardware_keycode = xev->detail;
 
           /* keyval is the key ignoring all modifiers ('1' vs. '!') */
@@ -928,12 +930,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->scroll.stage = stage;
 
             event->scroll.time = xev->time;
-            event->scroll.x = xev->event_x;
-            event->scroll.y = xev->event_y;
-            event->scroll.modifier_state =
-              _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                         &xev->buttons,
-                                                         &xev->group);
+            event->scroll.x = xev->event_x / window_scale;
+            event->scroll.y = xev->event_y / window_scale;
+	    _clutter_input_device_xi2_translate_state (event,
+						       &xev->mods,
+						       &xev->buttons,
+						       &xev->group);
 
             clutter_event_set_source_device (event, source_device);
             clutter_event_set_device (event, device);
@@ -976,13 +978,13 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->button.stage = stage;
 
             event->button.time = xev->time;
-            event->button.x = xev->event_x;
-            event->button.y = xev->event_y;
+            event->button.x = xev->event_x / window_scale;
+            event->button.y = xev->event_y / window_scale;
             event->button.button = xev->detail;
-            event->button.modifier_state =
-              _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                         &xev->buttons,
-                                                         &xev->group);
+	    _clutter_input_device_xi2_translate_state (event,
+						       &xev->mods,
+						       &xev->buttons,
+						       &xev->group);
 
             clutter_event_set_source_device (event, source_device);
             clutter_event_set_device (event, device);
@@ -1059,12 +1061,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
             event->scroll.stage = stage;
             event->scroll.time = xev->time;
-            event->scroll.x = xev->event_x;
-            event->scroll.y = xev->event_y;
-            event->scroll.modifier_state =
-              _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                         &xev->buttons,
-                                                         &xev->group);
+            event->scroll.x = xev->event_x / window_scale;
+            event->scroll.y = xev->event_y / window_scale;
+	    _clutter_input_device_xi2_translate_state (event,
+						       &xev->mods,
+						       &xev->buttons,
+						       &xev->group);
 
             clutter_event_set_scroll_delta (event, delta_x, delta_y);
             clutter_event_set_source_device (event, source_device);
@@ -1088,12 +1090,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         event->motion.stage = stage;
 
         event->motion.time = xev->time;
-        event->motion.x = xev->event_x;
-        event->motion.y = xev->event_y;
-        event->motion.modifier_state =
-          _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                     &xev->buttons,
-                                                     &xev->group);
+        event->motion.x = xev->event_x / window_scale;
+        event->motion.y = xev->event_y / window_scale;
+	_clutter_input_device_xi2_translate_state (event,
+						   &xev->mods,
+						   &xev->buttons,
+						   &xev->group);
 
         clutter_event_set_source_device (event, source_device);
         clutter_event_set_device (event, device);
@@ -1140,12 +1142,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         event->touch.stage = stage;
         event->touch.time = xev->time;
-        event->touch.x = xev->event_x;
-        event->touch.y = xev->event_y;
-        event->touch.modifier_state =
-          _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                     &xev->buttons,
-                                                     &xev->group);
+        event->touch.x = xev->event_x / window_scale;
+        event->touch.y = xev->event_y / window_scale;
+	_clutter_input_device_xi2_translate_state (event,
+						   &xev->mods,
+						   &xev->buttons,
+						   &xev->group);
 
         clutter_event_set_source_device (event, source_device);
 
@@ -1196,8 +1198,8 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         event->touch.stage = stage;
         event->touch.time = xev->time;
         event->touch.sequence = GUINT_TO_POINTER (xev->detail);
-        event->touch.x = xev->event_x;
-        event->touch.y = xev->event_y;
+        event->touch.x = xev->event_x / window_scale;
+        event->touch.y = xev->event_y / window_scale;
 
         clutter_event_set_source_device (event, source_device);
 
@@ -1211,10 +1213,10 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                             stage_x11,
                                             &xev->valuators);
 
-        event->touch.modifier_state =
-          _clutter_input_device_xi2_translate_state (&xev->mods,
-                                                     &xev->buttons,
-                                                     &xev->group);
+	_clutter_input_device_xi2_translate_state (event,
+						   &xev->mods,
+						   &xev->buttons,
+						   &xev->group);
         event->touch.modifier_state |= CLUTTER_BUTTON1_MASK;
 
         if (xev->flags & XITouchEmulatingPointer)
@@ -1254,8 +1256,8 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->crossing.related = NULL;
 
             event->crossing.time = xev->time;
-            event->crossing.x = xev->event_x;
-            event->crossing.y = xev->event_y;
+            event->crossing.x = xev->event_x / window_scale;
+            event->crossing.y = xev->event_y / window_scale;
 
             _clutter_input_device_set_stage (device, stage);
           }
@@ -1278,8 +1280,8 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->crossing.related = NULL;
 
             event->crossing.time = xev->time;
-            event->crossing.x = xev->event_x;
-            event->crossing.y = xev->event_y;
+            event->crossing.x = xev->event_x / window_scale;
+            event->crossing.y = xev->event_y / window_scale;
 
             _clutter_input_device_set_stage (device, NULL);
           }
